#include "wrapradio.h"

// Not wrapped:
// TAP_GetEvent, TAP_GetCurrentEvent, TAP_EPG_UpdateEvent, TAP_EPG_DeleteEvent, TAP_EPG_GetExtInfo

static bool (*unwrpRad_TAP_Hdd_GetRecInfo)( byte recSlot, TYPE_RecInfo *recInfo )= 0;
static bool (*unwrpRad_TAP_Hdd_GetPlayInfo)( TYPE_PlayInfo *playInfo )= 0;
static int  (*unwrpRad_TAP_Channel_GetTotalNum)( int *nTvSvc, int *nRadioSvc )= 0;
static int  (*unwrpRad_TAP_Channel_GetFirstInfo)( int svcType, TYPE_TapChInfo *chInfo )= 0;
static int  (*unwrpRad_TAP_Channel_GetNextInfo)( TYPE_TapChInfo *chInfo )= 0;
static int  (*unwrpRad_TAP_Channel_GetInfo)( int svcType, int svcNum, TYPE_TapChInfo *chInfo )= 0;
static int  (*unwrpRad_TAP_Channel_GetCurrent)( int *svcType, int *svcNum )= 0;
static int  (*unwrpRad_TAP_Channel_Start)(int mainSub, int svcType, int svcNum)= 0;
static int  (*unwrpRad_TAP_Channel_IsPlayable)( int mainSub, int svcType, int svcNum )= 0;
static TYPE_TapEvent*  (*unwrpRad_TAP_GetCurrentEvent)( int svcType, int svcNum )= 0;
static bool (*unwrpRad_TAP_Timer_GetInfo)( int entryNum, TYPE_TimerInfo *info )= 0;
static int  (*unwrpRad_TAP_Timer_Add)( TYPE_TimerInfo *info )= 0;
static int  (*unwrpRad_TAP_Timer_Modify)( int entryNum, TYPE_TimerInfo *info )= 0;

#define MAXLCN (999)

// Returns 1 if LCNs are all valid and unique across TV+Radio, 0 otherwise
static int ValidateLCNs()
{
  static int valid = -1;
  if(valid == -1)
  {
    byte inUse[MAXLCN+1];
    int num[2], type;
    valid = 0;
    memset(inUse, 0, MAXLCN+1);
    unwrpRad_TAP_Channel_GetTotalNum(&num[0], &num[1]);
    for(type=0; type<2; type++)
    {
      int i, max=num[type];
      for(i=0; i<max; i++)
      {
        TYPE_TapChInfo chInfo;
        // Error getting data? -- shouldn't happen
        if(!unwrpRad_TAP_Channel_GetInfo(type, i, &chInfo))
          return 0;
        // Bad LCN
        else if(chInfo.logicalChNum == 0 || chInfo.logicalChNum > MAXLCN)
          return 0;
        // Duplicate LCN
        else if(inUse[chInfo.logicalChNum])
          return 0;
        else
          inUse[chInfo.logicalChNum] = 1;
      }
    }
    valid = 1;
  }
  return valid;
}

static int NumTV()
{
  int numTV, numRadio;
  unwrpRad_TAP_Channel_GetTotalNum(&numTV, &numRadio);
  return numTV;
}

#define FakeTVCh(guard, elt) ({if(guard elt svcType != 0) { elt svcNum += NumTV(); elt svcType = 0;}})
#define UnfakeTVCh(elt) ({if(elt svcType == 0) { int num = NumTV(); if(elt svcNum >= num) { elt svcNum -= num; elt svcType = 1; }}})

static bool wrpRad_TAP_Hdd_GetRecInfo( byte recSlot, TYPE_RecInfo *recInfo )
{
  bool res = unwrpRad_TAP_Hdd_GetRecInfo(recSlot, recInfo);
  FakeTVCh(res &&, recInfo->);
  return res;
}

static bool wrpRad_TAP_Hdd_GetPlayInfo( TYPE_PlayInfo *playInfo )
{
  bool res = unwrpRad_TAP_Hdd_GetPlayInfo(playInfo);
  FakeTVCh(res &&, playInfo->);
  return res;
}

static int wrpRad_TAP_Channel_GetTotalNum( int *nTvSvc, int *nRadioSvc )
{
  int res = unwrpRad_TAP_Channel_GetTotalNum( nTvSvc, nRadioSvc );
  *nTvSvc += *nRadioSvc;
  *nRadioSvc = 0;
  return res;
}

static int GFISvcTV = -1;

static int wrpRad_TAP_Channel_GetFirstInfo( int svcType, TYPE_TapChInfo *chInfo )
{
  int res = 0;
  if(svcType == 0)
  {
    res = unwrpRad_TAP_Channel_GetFirstInfo( svcType, chInfo );
    GFISvcTV = 0;
  }
  else
    GFISvcTV = -1;
  return res;
}

static int wrpRad_TAP_Channel_GetNextInfo( TYPE_TapChInfo *chInfo )
{
  int res = 0;
  if(GFISvcTV >= 0)
  {
    res = unwrpRad_TAP_Channel_GetNextInfo( chInfo );
    if(res == 0 && GFISvcTV == 0)
    {
      res = unwrpRad_TAP_Channel_GetFirstInfo( 1, chInfo);
      GFISvcTV = 1;
    }
  }
  return res;
}

static int wrpRad_TAP_Channel_GetInfo( int svcType, int svcNum, TYPE_TapChInfo *chInfo )
{
  int res;
  UnfakeTVCh( );
  res = unwrpRad_TAP_Channel_GetInfo( svcType, svcNum, chInfo );
  return res;
}

static int wrpRad_TAP_Channel_GetCurrent( int *svcType, int *svcNum )
{
  int res = unwrpRad_TAP_Channel_GetCurrent( svcType, svcNum );
  FakeTVCh( ,*);
  return res;
}

static int wrpRad_TAP_Channel_Start(int mainSub, int svcType, int svcNum)
{
  int res;
  UnfakeTVCh( );
  res = unwrpRad_TAP_Channel_Start(mainSub, svcType, svcNum);
  return res;
}

static int wrpRad_TAP_Channel_IsPlayable( int mainSub, int svcType, int svcNum )
{
  int res;
  UnfakeTVCh( );
  res = unwrpRad_TAP_Channel_IsPlayable( mainSub, svcType, svcNum );
  return res;
}

static TYPE_TapEvent*  wrpRad_TAP_GetCurrentEvent( int svcType, int svcNum )
{
  TYPE_TapEvent *res;
  UnfakeTVCh( );
  res = unwrpRad_TAP_GetCurrentEvent( svcType, svcNum );
  return res;
}

static bool wrpRad_TAP_Timer_GetInfo( int entryNum, TYPE_TimerInfo *info )
{
  bool res = unwrpRad_TAP_Timer_GetInfo(entryNum, info);
  FakeTVCh(res &&, info->);
  return res;
}

static int wrpRad_TAP_Timer_Add( TYPE_TimerInfo *info )
{
  int res;
  UnfakeTVCh(info->);
  res = unwrpRad_TAP_Timer_Add(info);
  return res;
}

static int wrpRad_TAP_Timer_Modify( int entryNum, TYPE_TimerInfo *info )
{
  int res;
  UnfakeTVCh(info->);
  res = unwrpRad_TAP_Timer_Modify(entryNum, info);
  return res;
}

#define SETUNWRPRAD(api) unwrpRad_ ## api = api;
#define ADDWRAPPER(api)   api = wrpRad_ ## api;
#define REMWRAPPER(api)   api = unwrpRad_ ## api;

#define DOWORK(WORK) \
  WORK(TAP_Hdd_GetRecInfo); \
  WORK(TAP_Hdd_GetPlayInfo); \
  WORK(TAP_Channel_GetTotalNum); \
  WORK(TAP_Channel_GetFirstInfo); \
  WORK(TAP_Channel_GetNextInfo); \
  WORK(TAP_Channel_GetInfo); \
  WORK(TAP_Channel_GetCurrent); \
  WORK(TAP_Channel_Start); \
  WORK(TAP_Channel_IsPlayable); \
  WORK(TAP_GetCurrentEvent); \
  WORK(TAP_Timer_GetInfo); \
  WORK(TAP_Timer_Add); \
  WORK(TAP_Timer_Modify);

static void SetUnwrpRad()
{
  if(unwrpRad_TAP_Hdd_GetRecInfo == 0)
  {
    DOWORK(SETUNWRPRAD);
  }
}

int WrapRadio()
{
  SetUnwrpRad();
  if(!ValidateLCNs())
    return 0;
  DOWORK(ADDWRAPPER);
  return 1;
}

int UnwrapRadio()
{
  if(TAP_Hdd_GetRecInfo != wrpRad_TAP_Hdd_GetRecInfo)
    return 0;
  DOWORK(REMWRAPPER);
  return 1;
}

int IsRadio(int svcNum)
{
  return svcNum >= NumTV();
}
